from abc import ABC
from dataclasses import dataclass
from typing import List, Type, Dict

from src.core.enums import TagIncrement, TagReference, Action


@dataclass
class Affix:
    prefix: List[str] = None
    suffix: List[str] = None
    join_char: str = None

    def __init__(self, affix: Dict):
        self.prefix = affix.get("prefix", [])
        self.suffix = affix.get("suffix", [])
        self.join_char = affix.get("join_char", "/")


@dataclass
class Pattern:
    regex: str = None
    message: str = None
    example: str = None

    def __init__(self, pattern: Dict):
        self.regex = pattern.get("regex", "")
        self.message = pattern.get("message", "")
        self.example = pattern.get("example", "")


@dataclass
class Branch:
    name: str = None
    pattern: Pattern = None

    def __init__(self, branch: Dict):
        self.name = branch.get("name")
        self.pattern = Pattern(branch.get("pattern", {}))


@dataclass
class Parameters(ABC):
    pass


@dataclass
class DeleteBranchParameters:
    source: Branch = None
    pattern: Pattern = None


@dataclass
class MergeParameters:
    source: Type[Branch] = None
    target: List[Branch] = None
    allow_new_merge: bool = None


@dataclass
class MergeRequestParameters:
    source: Type[Branch] = None
    target: Type[Branch] = None
    mr_template: str = None


@dataclass
class TagParameters:
    reference: Type[TagReference] = None
    increment: Type[TagIncrement] = None
    target: List[Branch] = None


@dataclass
class Do:
    action: Type[Action] = None
    parameters: Type[Parameters] = None


@dataclass
class Execution:
    step: int = None
    do: Type[Do] = None


@dataclass
class Action:
    name: str = None
    alias: str = None
    execution: List[Execution] = None


@dataclass
class Command:
    name: str = None
    alias: str = None
    actions: List[Action] = None


@dataclass
class Init:
    execution: List[Execution] = None


@dataclass
class Template:
    init: Type[Init] = None
    commons: Dict = None
    commands: List[Command] = None



from dataclasses import dataclass


class PhoenixException(BaseException):
    message: str = None


@dataclass
class NotAGitRepoException(PhoenixException):
    message: str = "Not a git repo!"


@dataclass
class InvalidTemplateException(PhoenixException):
    message: str = "Invalid Phoenix template!"


@dataclass
class InvalidPatternException(PhoenixException):
    message: str = "Invalid pattern!"


@dataclass
class InvalidExecutionException(PhoenixException):
    message: str = "Invalid execution!"


@dataclass
class CommandNotFoundException(PhoenixException):
    command: str = None

    def __post_init__(self):
        self.message = f"Command {self.command} not found!"


@dataclass
class ActionNotFoundException(PhoenixException):
    action: str = None

    def __post_init__(self):
        self.message = f"Action {self.action} not found!"


@dataclass
class UnstagedFilesException(PhoenixException):
    message: str = (
        "You have unstaged files! Please commit them before proceeding."
    )


@dataclass
class BranchAheadException(PhoenixException):
    message: str = (
        "You are ahead of remote branch! Please push your "
        "changes before proceeding."
    )


@dataclass
class ShowHelpException(PhoenixException):
    message: str = None


@dataclass
class InvalidVariableException(PhoenixException):
    message: str = None


@dataclass
class MethodNotImplementedException(PhoenixException):
    message: str = None


@dataclass
class BranchAlreadyExistsException(PhoenixException):
    message: str = "Branch already exists"



from dataclasses import dataclass
from typing import List, Dict


@dataclass
class Execution:
    command: Dict = None
    action: Dict = None
    variables: Dict = None
    arguments: List[str] = None


@dataclass
class ActionExecution:
    variables: Dict = None
    parameters: Dict = None
    arguments: List[str] = None


@dataclass
class Choice:
    index: int = 0
    choice: str = None
    text: str = None


import abc
import logging
import sys

from colorlog import ColoredFormatter

from src.core.phoenix import BLACK
from src.core.phoenix import RED
from src.core.phoenix import GREEN
from src.core.phoenix import BROWN_ORANGE
from src.core.phoenix import BLUE
from src.core.phoenix import PURPLE
from src.core.phoenix import CYAN
from src.core.phoenix import LIGHT_GRAY
from src.core.phoenix import DARK_GRAY
from src.core.phoenix import LIGHT_RED
from src.core.phoenix import LIGHT_GREEN
from src.core.phoenix import YELLOW
from src.core.phoenix import LIGHT_BLUE
from src.core.phoenix import LIGHT_PURPLE
from src.core.phoenix import LIGHT_CYAN
from src.core.phoenix import WHITE
from src.core.phoenix import NC


INPUT = 5
logging.addLevelName(INPUT, 'INPUT')

# Configuring logger properties
formatter = ColoredFormatter(
    ("[%(log_color)s%(levelname)-8s%(reset)s] "
     "%(asctime)s - " +
     LIGHT_BLUE +
     "%(name)s" +
     NC +
     " - "
     "%(message)s"),
    datefmt=None,
    reset=True,
    log_colors={
        'INFO':     'cyan',
        "INPUT":    'blue',
        'DEBUG':    'green',
        'WARNING':  'yellow',
        'ERROR':    'red',
        'CRITICAL': 'bold_white,bg_red',
    },
    secondary_log_colors={},
    style='%'
)

ch = logging.StreamHandler()
ch.setFormatter(formatter)


class Logger(abc.ABC):

    @staticmethod
    def info(cls, msg):
        Logger._log(lvl=logging.INFO, cls=cls, msg=msg)

    @staticmethod
    def input(cls, msg):
        Logger._log(lvl=INPUT, cls=cls, msg=msg)

    @staticmethod
    def debug(cls, msg):
        Logger._log(lvl=logging.DEBUG, cls=cls, msg=msg)

    @staticmethod
    def warn(cls, msg):
        Logger._log(lvl=logging.WARN, cls=cls, msg=msg)

    @staticmethod
    def error(cls, msg):
        Logger._log(lvl=logging.ERROR, cls=cls, msg=msg)
        sys.exit()

    @staticmethod
    def critical(cls, msg):
        Logger._log(lvl=logging.CRITICAL, cls=cls, msg=msg)
        sys.exit()

    @staticmethod
    def _log(lvl, cls, msg):
        logger = logging.getLogger(cls.__name__)
        logger.setLevel(lvl)
        logger.addHandler(ch)
        logger.log(level=lvl, msg=msg)



"""
git-phoenix setup.py
"""
import pathlib
from setuptools import setup, find_packages

# The directory containing this file
HERE = pathlib.Path(__file__).parent

# The text of the README file
README = (HERE / "README.md").read_text()

setup(
    name="git-phoenix",
    version="1.0.4",
    description="A dynamic git branching tool",
    long_description=README,
    long_description_content_type="text/markdown",
    url="https://github.com/victoraugustofd/git-phoenix",
    author="victoraugustofd",
    license="MIT",
    packages=find_packages(),
    include_package_data=True,
    install_requires=[
        "GitPython",
        "regex",
        "argparse",
        "jsonschema==4.0.1",
        "colorlog",
        "questionary",
        "click==8.0.1",
    ],
    python_requires=">=3.7",
    entry_points={"console_scripts": ["git-phoenix=src.git_phoenix:main"]},
)



import inspect


def is_mod_function(mod, func):
    return inspect.isfunction(func) and inspect.getmodule(func) == mod


def list_functions(mod):
    return [
        func.__name__
        for func in mod.__dict__.values()
        if is_mod_function(mod, func)
    ]
